@@ -212,7 +212,8 @@ class Highs {
   HighsStatus presolve();
 
   /**
-   * @brief Run the solver, accounting for any multiple objective
+   * @brief Run the solver, applying file-based options and user
+   * scaling before optimization
    */
   HighsStatus run();
 
@@ -1290,7 +1291,16 @@ class Highs {
    */
   static void resetGlobalScheduler(bool blocking = false);
 
-  // Start of advanced methods for HiGHS MIP solver
+  // Start of advanced methods: only for internal use!
+
+  // Nested methods below Highs::run()
+  //
+  // See highs/HighsRun.md
+  HighsStatus optimizeHighs();
+  HighsStatus optimizeModel();
+  HighsStatus calledOptimizeModel();
+  // Used in MIP solver as minimal LP solve
+  HighsStatus optimizeLp();
 
   const HighsSimplexStats& getSimplexStats() const {
     return ekk_instance_.getSimplexStats();
@@ -1349,6 +1359,13 @@ class Highs {
     return ekk_instance_.primal_phase1_dual_;
   }
 
+  /**
+   * @brief Generalisation of getColName and getRowName. Advanced
+   * method: for HiGHS C++ and C API
+   */
+  HighsStatus getColOrRowName(const HighsLp& lp, const bool is_col,
+                              const HighsInt index, std::string& name) const;
+
   /**
    * @brief Development methods
    */
@@ -1525,7 +1542,6 @@ class Highs {
   HighsRanging ranging_;
   HighsIis iis_;
   std::vector<HighsObjectiveSolution> saved_objective_and_solution_;
-  HighsFiles files_;
 
   HighsPresolveStatus model_presolve_status_ =
       HighsPresolveStatus::kNotPresolved;
@@ -1554,7 +1570,6 @@ class Highs {
   bool written_log_header_ = false;
 
   void reportModelStats() const;
-  HighsStatus optimizeModel();
 
   void exactResizeModel() {
     this->model_.lp_.exactResize();
@@ -1760,9 +1775,12 @@ class Highs {
   void restoreInfCost(HighsStatus& return_status);
   HighsStatus optionChangeAction();
 
+  HighsStatus userScale(HighsUserScaleData& data);
+  HighsStatus userUnscale(HighsUserScaleData& data);
   HighsStatus userScaleModel(HighsUserScaleData& data);
   HighsStatus userScaleSolution(HighsUserScaleData& data,
                                 bool update_kkt = false);
+
   HighsStatus computeIllConditioning(HighsIllConditioning& ill_conditioning,
                                      const bool constraint,
                                      const HighsInt method,
@@ -1782,10 +1800,6 @@ class Highs {
 
   bool tryPdlpCleanup(HighsInt& pdlp_cleanup_iteration_limit,
                       const HighsInfo& presolved_lp_info) const;
-
-  bool optionsHasHighsFiles() const;
-  void saveHighsFiles();
-  void getHighsFiles();
 };
 
 // Start of deprecated methods not in the Highs class
