@@ -17,7 +17,7 @@
 #include <exception>
 #include <map>
 
-#include "../extern/filereaderlp/reader.hpp"
+#include "io/filereaderlp/reader.hpp"
 #include "lp_data/HighsLpUtils.h"
 
 FilereaderRetcode FilereaderLp::readModelFromFile(const HighsOptions& options,
@@ -291,7 +291,7 @@ FilereaderRetcode FilereaderLp::readModelFromFile(const HighsOptions& options,
                    sum_num_zero > 1 ? "s" : "");
 
   } catch (std::invalid_argument& ex) {
-    // lpassert in extern/filereaderlp/def.hpp throws
+    // lpassert in highs/io/filereaderlp/def.hpp throws
     // std::invalid_argument whatever the error. Hence, unless
     // something is done specially - here or elsewhere -
     // FilereaderRetcode::kParserError will be returned.
