@@ -27,10 +27,14 @@ void highsLogHeader(const HighsLogOptions& log_options,
 
 #ifdef HIPO
 #ifdef BLAS_LIBRARIES
-  highsLogUser(log_options, HighsLogType::kInfo, "Using blas: %s\n",
+  highsLogUser(log_options, HighsLogType::kInfo, "Using BLAS: %s \n",
                BLAS_LIBRARIES);
 #else
-  highsLogUser(log_options, HighsLogType::kInfo, "Using blas: unknown\n");
+#ifdef HIPO_USES_OPENBLAS
+  highsLogUser(log_options, HighsLogType::kInfo, "Using BLAS: OpenBLAS \n");
+#else
+  highsLogUser(log_options, HighsLogType::kInfo, "Using BLAS: unknown \n");
+#endif
 #endif
 #endif
 }
