@@ -9,15 +9,16 @@ class HybridSolveHandler : public SolveHandler {
   const std::vector<std::vector<Int>>& swaps_;
   const std::vector<std::vector<double>>& pivot_2x2_;
 
+ public:
   void forwardSolve(std::vector<double>& x) const override;
   void backwardSolve(std::vector<double>& x) const override;
   void diagSolve(std::vector<double>& x) const override;
 
- public:
   HybridSolveHandler(const Symbolic& S,
                      const std::vector<std::vector<double>>& sn_columns,
                      const std::vector<std::vector<Int>>& swaps,
-                     const std::vector<std::vector<double>>& pivot_2x2);
+                     const std::vector<std::vector<double>>& pivot_2x2,
+                     DataCollector& data);
 };
 
 }  // namespace hipo
