@@ -9,8 +9,8 @@
 namespace hipo {
 
 FactorHiGHSSolver::FactorHiGHSSolver(Options& options, const Model& model,
-                                     const Regularisation& regul, Info* info,
-                                     IpmData* record, const LogHighs& log)
+                                     const Regularisation& regul, Info& info,
+                                     IpmData& record, const LogHighs& log)
     : FH_(&log, options.block_size),
       S_{},
       regul_{regul},
@@ -18,6 +18,10 @@ FactorHiGHSSolver::FactorHiGHSSolver(Options& options, const Model& model,
       data_{record},
       log_{log},
       model_{model},
+      A_{model.A()},
+      mA_{A_.num_row_},
+      nA_{A_.num_col_},
+      nzA_{A_.numNz()},
       options_{options} {}
 
 void FactorHiGHSSolver::clear() {
@@ -25,130 +29,91 @@ void FactorHiGHSSolver::clear() {
   FH_.newIter();
 }
 
-static Int getASstructure(const HighsSparseMatrix& A, std::vector<Int>& ptr,
-                          std::vector<Int>& rows) {
-  // Augmented system structure
+// =========================================================================
+// Build structure and values of matrices
+// =========================================================================
 
-  Int nA = A.num_col_;
-  Int mA = A.num_row_;
-  Int nzA = A.numNz();
+Int FactorHiGHSSolver::buildASstructure(Int64 nz_limit) {
+  // Build lower triangular structure of the augmented system.
+  // Build values of AS that will not change during the iterations.
 
-  if (nA + nzA + mA > kHighsIInf) return kStatusOoM;
+  log_.printDevInfo("Building AS structure\n");
 
-  ptr.resize(nA + mA + 1);
-  rows.resize(nA + nzA + mA);
+  // AS matrix must fit into HighsInt
+  if ((Int64)nA_ + mA_ + nzA_ > nz_limit) return kStatusOverflow;
+
+  ptrAS_.resize(nA_ + mA_ + 1);
+  rowsAS_.resize(nA_ + nzA_ + mA_);
+  valAS_.resize(nA_ + nzA_ + mA_);
 
   Int next = 0;
 
-  for (Int i = 0; i < nA; ++i) {
+  for (Int i = 0; i < nA_; ++i) {
     // diagonal element
-    rows[next] = i;
+    rowsAS_[next] = i;
     ++next;
 
     // column of A
-    for (Int el = A.start_[i]; el < A.start_[i + 1]; ++el) {
-      rows[next] = A.index_[el] + nA;
+    for (Int el = A_.start_[i]; el < A_.start_[i + 1]; ++el) {
+      rowsAS_[next] = nA_ + A_.index_[el];
+      valAS_[next] = A_.value_[el];  // values of AS that will not change
       ++next;
     }
 
-    ptr[i + 1] = next;
+    ptrAS_[i + 1] = next;
   }
 
   // 2,2 block
-  for (Int i = 0; i < mA; ++i) {
-    rows[next] = nA + i;
+  for (Int i = 0; i < mA_; ++i) {
+    rowsAS_[next] = nA_ + i;
     ++next;
-    ptr[nA + i + 1] = ptr[nA + i] + 1;
+    ptrAS_[nA_ + i + 1] = ptrAS_[nA_ + i] + 1;
   }
 
   return kStatusOk;
 }
 
-Int FactorHiGHSSolver::setup() {
-  if (Int status = setNla()) return status;
-  setParallel();
+Int FactorHiGHSSolver::buildASvalues(const std::vector<double>& scaling) {
+  // build AS values that change during iterations.
 
-  S_.print(log_, log_.debug(1));
-  return kStatusOk;
-}
+  assert(!ptrAS_.empty() && !rowsAS_.empty());
 
-Int FactorHiGHSSolver::buildNEvalues(const HighsSparseMatrix& A,
-                                     const std::vector<double>& scaling) {
-  // given the NE structure already computed, fill in the NE values
-
-  assert(!ptrNE_.empty() && !rowsNE_.empty());
-
-  valNE_.assign(rowsNE_.size(), 0.0);
-
-  std::vector<double> work(A.num_row_, 0.0);
-
-  for (Int row = 0; row < A.num_row_; ++row) {
-    // go along the entries of the row, and then down each column.
-    // this builds the lower triangular part of the row-th column of AAt.
-
-    for (Int el = ptrNE_rw_[row]; el < ptrNE_rw_[row + 1]; ++el) {
-      Int col = idxNE_rw_[el];
-      Int corr = corr_NE_[el];
-
-      const double theta =
-          scaling.empty() ? 1.0 : 1.0 / (scaling[col] + regul_.primal);
-
-      const double row_value = theta * A.value_[corr];
-
-      // for each nonzero in the row, go down corresponding column, starting
-      // from current position
-      for (Int colEl = corr; colEl < A.start_[col + 1]; ++colEl) {
-        Int row2 = A.index_[colEl];
-
-        // row2 is guaranteed to be larger or equal than row
-        // (provided that the columns of A are sorted)
-
-        // compute and accumulate value
-        double value = row_value * A.value_[colEl];
-        work[row2] += value;
-      }
-    }
-    // intersection of row with rows below finished.
-
-    // read from work, using indices of column "row" of AAt
-    for (Int el = ptrNE_[row]; el < ptrNE_[row + 1]; ++el) {
-      Int index = rowsNE_[el];
-      valNE_[el] = work[index];
-      work[index] = 0.0;
-    }
+  for (Int i = 0; i < nA_; ++i) {
+    valAS_[ptrAS_[i]] = -scaling[i];
   }
 
   return kStatusOk;
 }
 
-Int FactorHiGHSSolver::buildNEstructure(const HighsSparseMatrix& A,
-                                        int64_t nz_limit) {
+Int FactorHiGHSSolver::buildNEstructure(Int64 nz_limit) {
   // Build lower triangular structure of AAt.
   // This approach uses a column-wise copy of A, a partial row-wise copy and a
   // vector of corresponding indices.
 
   // NB: A must have sorted columns for this to work
 
+  log_.printDevInfo("Building NE structure\n");
+
   // create partial row-wise representation without values, and array or
   // corresponding indices between cw and rw representation
   {
-    ptrNE_rw_.assign(A.num_row_ + 1, 0);
-    idxNE_rw_.assign(A.numNz(), 0);
+    ptrA_rw_.assign(mA_ + 1, 0);
+    idxA_rw_.assign(nzA_, 0);
 
     // pointers of row-start
-    for (Int el = 0; el < A.numNz(); ++el) ptrNE_rw_[A.index_[el] + 1]++;
-    for (Int i = 0; i < A.num_row_; ++i) ptrNE_rw_[i + 1] += ptrNE_rw_[i];
+    for (Int el = 0; el < nzA_; ++el) ptrA_rw_[A_.index_[el] + 1]++;
+    for (Int i = 0; i < mA_; ++i) ptrA_rw_[i + 1] += ptrA_rw_[i];
 
-    std::vector<Int> temp = ptrNE_rw_;
-    corr_NE_.assign(A.numNz(), 0);
+    std::vector<Int> temp = ptrA_rw_;
+    corr_A_.assign(nzA_, 0);
 
     // rw-indices and corresponding indices created together
-    for (Int col = 0; col < A.num_col_; ++col) {
-      for (Int el = A.start_[col]; el < A.start_[col + 1]; ++el) {
-        Int row = A.index_[el];
+    for (Int col = 0; col < nA_; ++col) {
+      for (Int el = A_.start_[col]; el < A_.start_[col + 1]; ++el) {
+        Int row = A_.index_[el];
 
-        corr_NE_[temp[row]] = el;
-        idxNE_rw_[temp[row]] = col;
+        corr_A_[temp[row]] = el;
+        idxA_rw_[temp[row]] = col;
         temp[row]++;
       }
     }
@@ -158,28 +123,28 @@ Int FactorHiGHSSolver::buildNEstructure(const HighsSparseMatrix& A,
   rowsNE_.clear();
 
   // ptr is allocated its exact size
-  ptrNE_.resize(A.num_row_ + 1, 0);
+  ptrNE_.resize(mA_ + 1, 0);
 
   // keep track if given entry is nonzero, in column considered
-  std::vector<bool> is_nz(A.num_row_, false);
+  std::vector<bool> is_nz(mA_, false);
 
   // temporary storage of indices
-  std::vector<Int> temp_index(A.num_row_);
+  std::vector<Int> temp_index(mA_);
 
-  for (Int row = 0; row < A.num_row_; ++row) {
+  for (Int row = 0; row < mA_; ++row) {
     // go along the entries of the row, and then down each column.
     // this builds the lower triangular part of the row-th column of AAt.
 
     Int nz_in_col = 0;
 
-    for (Int el = ptrNE_rw_[row]; el < ptrNE_rw_[row + 1]; ++el) {
-      Int col = idxNE_rw_[el];
-      Int corr = corr_NE_[el];
+    for (Int el = ptrA_rw_[row]; el < ptrA_rw_[row + 1]; ++el) {
+      Int col = idxA_rw_[el];
+      Int corr = corr_A_[el];
 
       // for each nonzero in the row, go down corresponding column, starting
       // from current position
-      for (Int colEl = corr; colEl < A.start_[col + 1]; ++colEl) {
-        Int row2 = A.index_[colEl];
+      for (Int colEl = corr; colEl < A_.start_[col + 1]; ++colEl) {
+        Int row2 = A_.index_[colEl];
 
         // row2 is guaranteed to be larger or equal than row
         // (provided that the columns of A are sorted)
@@ -194,9 +159,8 @@ Int FactorHiGHSSolver::buildNEstructure(const HighsSparseMatrix& A,
     }
     // intersection of row with rows below finished.
 
-    // if the total number of nonzeros exceeds the maximum, return error
-    if ((int64_t)ptrNE_[row] + (int64_t)nz_in_col >= nz_limit)
-      return kStatusOoM;
+    // if the total number of nonzeros exceeds the maximum, return error.
+    if ((Int64)ptrNE_[row] + nz_in_col >= nz_limit) return kStatusOverflow;
 
     // update pointers
     ptrNE_[row + 1] = ptrNE_[row] + nz_in_col;
@@ -213,6 +177,96 @@ Int FactorHiGHSSolver::buildNEstructure(const HighsSparseMatrix& A,
   return kStatusOk;
 }
 
+Int FactorHiGHSSolver::buildNEvalues(const std::vector<double>& scaling) {
+  // given the NE structure already computed, fill in the NE values
+
+  assert(!ptrNE_.empty() && !rowsNE_.empty());
+
+  valNE_.resize(rowsNE_.size());
+
+  std::vector<double> work(mA_, 0.0);
+
+  for (Int row = 0; row < mA_; ++row) {
+    // go along the entries of the row, and then down each column.
+    // this builds the lower triangular part of the row-th column of AAt.
+
+    for (Int el = ptrA_rw_[row]; el < ptrA_rw_[row + 1]; ++el) {
+      Int col = idxA_rw_[el];
+      Int corr = corr_A_[el];
+
+      const double theta =
+          scaling.empty() ? 1.0 : 1.0 / (scaling[col] + regul_.primal);
+
+      const double row_value = theta * A_.value_[corr];
+
+      // for each nonzero in the row, go down corresponding column, starting
+      // from current position
+      for (Int colEl = corr; colEl < A_.start_[col + 1]; ++colEl) {
+        Int row2 = A_.index_[colEl];
+
+        // row2 is guaranteed to be larger or equal than row
+        // (provided that the columns of A are sorted)
+
+        // compute and accumulate value
+        double value = row_value * A_.value_[colEl];
+        work[row2] += value;
+      }
+    }
+    // intersection of row with rows below finished.
+
+    // read from work, using indices of column "row" of AAt
+    for (Int el = ptrNE_[row]; el < ptrNE_[row + 1]; ++el) {
+      Int index = rowsNE_[el];
+      valNE_[el] = work[index];
+      work[index] = 0.0;
+    }
+  }
+
+  return kStatusOk;
+}
+
+// =========================================================================
+// Analyse phase
+// =========================================================================
+
+Int FactorHiGHSSolver::analyseAS(Symbolic& S) {
+  // Perform analyse phase of augmented system and return symbolic factorisation
+  // in object S and the status.
+
+  Clock clock;
+  if (Int status = buildASstructure()) return status;
+  info_.matrix_structure_time = clock.stop();
+
+  // create vector of signs of pivots
+  std::vector<Int> pivot_signs(nA_ + mA_, -1);
+  for (Int i = 0; i < mA_; ++i) pivot_signs[nA_ + i] = 1;
+
+  log_.printDevInfo("Performing AS analyse phase\n");
+
+  return chooseOrdering(rowsAS_, ptrAS_, pivot_signs, S);
+}
+
+Int FactorHiGHSSolver::analyseNE(Symbolic& S, Int64 nz_limit) {
+  // Perform analyse phase of augmented system and return symbolic factorisation
+  // in object S and the status. If building the matrix failed, the status is
+  // set to kStatusOverflow.
+
+  Clock clock;
+  if (Int status = buildNEstructure(nz_limit)) return status;
+  info_.matrix_structure_time = clock.stop();
+
+  // create vector of signs of pivots
+  std::vector<Int> pivot_signs(mA_, 1);
+
+  log_.printDevInfo("Performing NE analyse phase\n");
+
+  return chooseOrdering(rowsNE_, ptrNE_, pivot_signs, S);
+}
+
+// =========================================================================
+// Factorise phase
+// =========================================================================
+
 Int FactorHiGHSSolver::factorAS(const HighsSparseMatrix& A,
                                 const std::vector<double>& scaling) {
   // only execute factorisation if it has not been done yet
@@ -220,55 +274,18 @@ Int FactorHiGHSSolver::factorAS(const HighsSparseMatrix& A,
 
   Clock clock;
 
-  // initialise
-  std::vector<Int> ptrLower;
-  std::vector<Int> rowsLower;
-  std::vector<double> valLower;
-
-  Int nA = A.num_col_;
-  Int mA = A.num_row_;
-  Int nzA = A.numNz();
-
-  ptrLower.resize(nA + mA + 1);
-  rowsLower.resize(nA + nzA + mA);
-  valLower.resize(nA + nzA + mA);
-
-  // build lower triangle
-  Int next = 0;
-
-  for (Int i = 0; i < nA; ++i) {
-    // diagonal element
-    rowsLower[next] = i;
-    valLower[next++] = -scaling[i];
-
-    // column of A
-    for (Int el = A.start_[i]; el < A.start_[i + 1]; ++el) {
-      rowsLower[next] = A.index_[el] + nA;
-      valLower[next++] = A.value_[el];
-    }
-
-    ptrLower[i + 1] = next;
-  }
-
-  // 2,2 block
-  for (Int i = 0; i < mA; ++i) {
-    rowsLower[next] = nA + i;
-    valLower[next++] = 0.0;
-    ptrLower[nA + i + 1] = ptrLower[nA + i] + 1;
-  }
-  if (info_) info_->matrix_time += clock.stop();
+  // build matrix
+  buildASvalues(scaling);
+  info_.matrix_time += clock.stop();
 
   // set static regularisation, since it may have changed
   FH_.setRegularisation(regul_.primal, regul_.dual);
 
   // factorise matrix
   clock.start();
-  if (FH_.factorise(S_, rowsLower, ptrLower, valLower))
-    return kStatusErrorFactorise;
-  if (info_) {
-    info_->factor_time += clock.stop();
-    info_->factor_number++;
-  }
+  if (FH_.factorise(S_, rowsAS_, ptrAS_, valAS_)) return kStatusErrorFactorise;
+  info_.factor_time += clock.stop();
+  info_.factor_number++;
 
   this->valid_ = true;
   return kStatusOk;
@@ -281,53 +298,27 @@ Int FactorHiGHSSolver::factorNE(const HighsSparseMatrix& A,
 
   Clock clock;
 
-  Int nA = A.num_col_;
-  Int mA = A.num_row_;
-  Int nzA = A.numNz();
-
   // build matrix
-  Int status = buildNEvalues(A, scaling);
-  if (info_) info_->matrix_time += clock.stop();
+  buildNEvalues(scaling);
+  info_.matrix_time += clock.stop();
 
   // set static regularisation, since it may have changed
   FH_.setRegularisation(regul_.primal, regul_.dual);
 
   // factorise
   clock.start();
-  // make copies of structure, because factorise object will take ownership and
-  // modify them.
-  std::vector<Int> ptrNE(ptrNE_);
-  std::vector<Int> rowsNE(rowsNE_);
-  if (FH_.factorise(S_, rowsNE, ptrNE, valNE_)) return kStatusErrorFactorise;
-  if (info_) {
-    info_->factor_time += clock.stop();
-    info_->factor_number++;
-  }
+  if (FH_.factorise(S_, rowsNE_, ptrNE_, valNE_)) return kStatusErrorFactorise;
+
+  info_.factor_time += clock.stop();
+  info_.factor_number++;
 
   this->valid_ = true;
   return kStatusOk;
 }
 
-Int FactorHiGHSSolver::solveNE(const std::vector<double>& rhs,
-                               std::vector<double>& lhs) {
-  // only execute the solve if factorisation is valid
-  assert(this->valid_);
-
-  // initialise lhs with rhs
-  lhs = rhs;
-
-  Clock clock;
-  if (FH_.solve(lhs)) return kStatusErrorSolve;
-  if (info_) {
-    info_->solve_time += clock.stop();
-    info_->solve_number++;
-  }
-  if (data_) {
-    data_->back().num_solves++;
-  }
-
-  return kStatusOk;
-}
+// =========================================================================
+// Solve phase
+// =========================================================================
 
 Int FactorHiGHSSolver::solveAS(const std::vector<double>& rhs_x,
                                const std::vector<double>& rhs_y,
@@ -345,13 +336,11 @@ Int FactorHiGHSSolver::solveAS(const std::vector<double>& rhs_x,
 
   Clock clock;
   if (FH_.solve(rhs)) return kStatusErrorSolve;
-  if (info_) {
-    info_->solve_time += clock.stop();
-    info_->solve_number++;
-  }
-  if (data_) {
-    data_->back().num_solves++;
-  }
+
+  info_.solve_time += clock.stop();
+  info_.solve_number++;
+
+  data_.back().num_solves++;
 
   // split lhs
   lhs_x = std::vector<double>(rhs.begin(), rhs.begin() + n);
@@ -360,81 +349,43 @@ Int FactorHiGHSSolver::solveAS(const std::vector<double>& rhs_x,
   return kStatusOk;
 }
 
-double FactorHiGHSSolver::flops() const { return S_.flops(); }
-double FactorHiGHSSolver::spops() const { return S_.spops(); }
-double FactorHiGHSSolver::nz() const { return (double)S_.nz(); }
-void FactorHiGHSSolver::getReg(std::vector<double>& reg) {
-  return FH_.getRegularisation(reg);
-}
+Int FactorHiGHSSolver::solveNE(const std::vector<double>& rhs,
+                               std::vector<double>& lhs) {
+  // only execute the solve if factorisation is valid
+  assert(this->valid_);
 
-Int FactorHiGHSSolver::analyseAS(Symbolic& S) {
-  // Perform analyse phase of augmented system and return symbolic factorisation
-  // in object S and the status.
-
-  log_.printDevInfo("Building AS structure\n");
+  // initialise lhs with rhs
+  lhs = rhs;
 
   Clock clock;
-  std::vector<Int> ptrLower, rowsLower;
-  if (Int status = getASstructure(model_.A(), ptrLower, rowsLower))
-    return status;
-  if (info_) info_->matrix_structure_time = clock.stop();
+  if (FH_.solve(lhs)) return kStatusErrorSolve;
 
-  // create vector of signs of pivots
-  std::vector<Int> pivot_signs(model_.A().num_col_ + model_.A().num_row_, -1);
-  for (Int i = 0; i < model_.A().num_row_; ++i)
-    pivot_signs[model_.A().num_col_ + i] = 1;
+  info_.solve_time += clock.stop();
+  info_.solve_number++;
 
-  log_.printDevInfo("Performing AS analyse phase\n");
+  data_.back().num_solves++;
 
-  clock.start();
-  Int status = FH_.analyse(S, rowsLower, ptrLower, pivot_signs);
-  if (info_) info_->analyse_AS_time = clock.stop();
-
-  if (status && log_.debug(2)) {
-    log_.print("Failed augmented system:");
-    S.print(log_, true);
-  }
-
-  return status ? kStatusErrorAnalyse : kStatusOk;
+  return kStatusOk;
 }
 
-void FactorHiGHSSolver::freeNEmemory() {
-  // Swap NE data structures with empty vectors, to guarantee that memory is
-  // freed.
+// =========================================================================
+// Automatic selections
+// =========================================================================
 
-  std::vector<Int>().swap(ptrNE_);
-  std::vector<Int>().swap(rowsNE_);
-  std::vector<Int>().swap(ptrNE_rw_);
-  std::vector<Int>().swap(idxNE_rw_);
-  std::vector<Int>().swap(corr_NE_);
-}
+Int FactorHiGHSSolver::setup() {
+  if (Int status = setNla()) return status;
+  setParallel();
 
-Int FactorHiGHSSolver::analyseNE(Symbolic& S, int64_t nz_limit) {
-  // Perform analyse phase of augmented system and return symbolic factorisation
-  // in object S and the status. If building the matrix failed, the status is
-  // set to OoM.
+  S_.print(log_, log_.debug(1));
 
-  log_.printDevInfo("Building NE structure\n");
-
-  Clock clock;
-  if (Int status = buildNEstructure(model_.A(), nz_limit)) return status;
-  if (info_) info_->matrix_structure_time = clock.stop();
-
-  // create vector of signs of pivots
-  std::vector<Int> pivot_signs(model_.A().num_row_, 1);
-
-  log_.printDevInfo("Performing NE analyse phase\n");
-
-  clock.start();
-  Int status = FH_.analyse(S, rowsNE_, ptrNE_, pivot_signs);
-  if (info_) info_->analyse_NE_time = clock.stop();
-
-  if (status && log_.debug(2)) {
-    log_.print("Failed normal equations:");
-    S.print(log_, true);
+  // Warn about large memory consumption
+  if (S_.storage() > kLargeStorageGB * 1024 * 1024 * 1024) {
+    log_.printw("Large amount of memory required\n");
   }
 
-  return status ? kStatusErrorAnalyse : kStatusOk;
+  log_.print("\n");
+
+  return kStatusOk;
 }
 
 Int FactorHiGHSSolver::chooseNla() {
@@ -446,14 +397,18 @@ Int FactorHiGHSSolver::chooseNla() {
   Symbolic symb_AS{};
   bool failure_NE = false;
   bool failure_AS = false;
-
-  symb_NE.setMetisNo2hop(options_.metis_no2hop);
-  symb_AS.setMetisNo2hop(options_.metis_no2hop);
+  bool overflow_NE = false;
+  bool overflow_AS = false;
 
   Clock clock;
 
   // Perform analyse phase of augmented system
-  if (analyseAS(symb_AS)) failure_AS = true;
+  Int AS_status = analyseAS(symb_AS);
+  if (AS_status) failure_AS = true;
+  if (AS_status == kStatusOverflow) {
+    log_.printDevInfo("Integer overflow forming AS matrix\n");
+    overflow_AS = true;
+  }
 
   // Perform analyse phase of normal equations
   if (model_.m() > kMinRowsForDensity &&
@@ -464,12 +419,15 @@ Int FactorHiGHSSolver::chooseNla() {
   } else {
     // If NE has more nonzeros than the factor of AS, then it's likely that AS
     // will be preferred, so stop computation of NE.
-    int64_t NE_nz_limit = symb_AS.nz() * kSymbNzMult;
+    Int64 NE_nz_limit = symb_AS.nz() * kSymbNzMult;
     if (failure_AS || NE_nz_limit > kHighsIInf) NE_nz_limit = kHighsIInf;
 
     Int NE_status = analyseNE(symb_NE, NE_nz_limit);
     if (NE_status) failure_NE = true;
-    if (NE_status == kStatusOoM) log_.printDevInfo("NE matrix is too large\n");
+    if (NE_status == kStatusOverflow) {
+      log_.printDevInfo("Integer overflow forming NE matrix\n");
+      overflow_NE = true;
+    }
   }
 
   Int status = kStatusOk;
@@ -484,13 +442,12 @@ Int FactorHiGHSSolver::chooseNla() {
     options_.nla = kOptionNlaNormEq;
     log_stream << textline("Newton system:") << "NE preferred (AS failed)\n";
   } else if (failure_AS && failure_NE) {
-    status = kStatusErrorAnalyse;
+    if (overflow_AS && overflow_NE)
+      status = kStatusOverflow;
+    else
+      status = kStatusErrorAnalyse;
+
     log_.printe("Both NE and AS failed analyse phase\n");
-    if ((symb_AS.fillin() > 50 || symb_NE.fillin() > 50) &&
-        !options_.metis_no2hop)
-      log_.print(
-          "Large fill-in in factorisation. Consider setting the "
-          "hipo_metis_no2hop option to true\n");
   } else {
     // Total number of operations, given by dense flops and sparse indexing
     // operations, weighted with an empirical factor
@@ -520,23 +477,117 @@ Int FactorHiGHSSolver::chooseNla() {
 
   log_.print(log_stream);
 
-  if (status != kStatusErrorAnalyse) {
+  if (status == kStatusOk) {
     if (options_.nla == kOptionNlaAugmented) {
       S_ = std::move(symb_AS);
       freeNEmemory();
     } else {
       S_ = std::move(symb_NE);
+      freeASmemory();
     }
   }
 
   return status;
 }
 
+Int FactorHiGHSSolver::chooseOrdering(const std::vector<Int>& rows,
+                                      const std::vector<Int>& ptr,
+                                      const std::vector<Int>& signs,
+                                      Symbolic& S) {
+  // Run analyse phase.
+  // - If ordering is "amd", "metis", "rcm" run only the ordering requested.
+  // - If ordering is "choose", run "amd", "metis", and choose the best.
+
+  Clock clock;
+
+  // select which fill-reducing orderings should be tried
+  std::vector<std::string> orderings_to_try;
+  if (options_.ordering != kHighsChooseString)
+    orderings_to_try.push_back(options_.ordering);
+  else {
+    orderings_to_try.push_back("amd");
+    orderings_to_try.push_back("metis");
+    // rcm is much worse in general, so no point in trying for now
+  }
+
+  std::vector<Symbolic> symbolics(orderings_to_try.size(), S);
+  std::vector<bool> status(orderings_to_try.size(), 0);
+  Int num_success = 0;
+
+  for (Int i = 0; i < orderings_to_try.size(); ++i) {
+    clock.start();
+    status[i] =
+        FH_.analyse(symbolics[i], rows, ptr, signs, orderings_to_try[i]);
+    info_.analyse_AS_time += clock.stop();
+
+    if (status[i] && log_.debug(2)) {
+      log_.print("Failed symbolic:");
+      symbolics[i].print(log_, true);
+    }
+
+    if (!status[i]) ++num_success;
+  }
+
+  if (orderings_to_try.size() < 2) {
+    S = std::move(symbolics[0]);
+
+  } else if (orderings_to_try.size() == 2) {
+    // if there's only one success, obvious choice
+    if (status[0] && !status[1])
+      S = std::move(symbolics[1]);
+    else if (!status[0] && status[1])
+      S = std::move(symbolics[0]);
+
+    else if (num_success > 1) {
+      // need to choose the better ordering
+
+      const double flops_0 = symbolics[0].flops();
+      const double flops_1 = symbolics[1].flops();
+      const double sn_avg_0 = symbolics[0].size() / symbolics[0].sn();
+      const double sn_avg_1 = symbolics[1].size() / symbolics[1].sn();
+      const double bytes_0 = symbolics[0].storage();
+      const double bytes_1 = symbolics[1].storage();
+
+      Int chosen = -1;
+
+      // selection rule:
+      // - if flops have a clear winner (+/- 20%), then choose it.
+      // - otherwise, choose the one with larger supernodes.
+
+      if (flops_0 > kFlopsOrderingThresh * flops_1)
+        chosen = 1;
+      else if (flops_1 > kFlopsOrderingThresh * flops_0)
+        chosen = 0;
+      else if (sn_avg_0 > sn_avg_1)
+        chosen = 0;
+      else
+        chosen = 1;
+
+      // fix selection if one or more require too much memory
+      const double bytes_thresh = kLargeStorageGB * 1024 * 1024 * 1024;
+      if (bytes_0 > bytes_thresh || bytes_1 > bytes_thresh) {
+        if (bytes_0 > bytes_1)
+          chosen = 1;
+        else
+          chosen = 0;
+      }
+
+      assert(chosen == 0 || chosen == 1);
+
+      S = std::move(symbolics[chosen]);
+    }
+
+  } else {
+    // only two orderings tried for now
+    assert(0 == 1);
+  }
+
+  return num_success > 0 ? kStatusOk : kStatusErrorAnalyse;
+}
+
 Int FactorHiGHSSolver::setNla() {
   std::stringstream log_stream;
 
-  S_.setMetisNo2hop(options_.metis_no2hop);
-
   switch (options_.nla) {
     case kOptionNlaAugmented: {
       if (analyseAS(S_)) {
@@ -549,9 +600,9 @@ Int FactorHiGHSSolver::setNla() {
 
     case kOptionNlaNormEq: {
       Int status = analyseNE(S_);
-      if (status == kStatusOoM) {
-        log_.printe("NE requested, matrix is too large\n");
-        return kStatusOoM;
+      if (status == kStatusOverflow) {
+        log_.printe("NE requested, integer overflow\n");
+        return kStatusOverflow;
       } else if (status) {
         log_.printe("NE requested, failed analyse phase\n");
         return kStatusErrorAnalyse;
@@ -619,6 +670,13 @@ void FactorHiGHSSolver::setParallel() {
       }
 #endif
 
+      // If serial memory is too large, switch off tree parallelism to avoid
+      // running out of memory
+      double num_GB = S_.storage() / 1024 / 1024 / 1024;
+      if (num_GB > kLargeStorageGB) {
+        parallel_tree = false;
+      }
+
       if (parallel_tree && parallel_node) {
         options_.parallel = kOptionParallelOn;
         log_stream << "Full preferred\n";
@@ -649,4 +707,32 @@ void FactorHiGHSSolver::setParallel() {
   S_.setParallel(parallel_tree, parallel_node);
 }
 
+// =========================================================================
+// Other stuff
+// =========================================================================
+
+double FactorHiGHSSolver::flops() const { return S_.flops(); }
+double FactorHiGHSSolver::spops() const { return S_.spops(); }
+double FactorHiGHSSolver::nz() const { return (double)S_.nz(); }
+void FactorHiGHSSolver::getReg(std::vector<double>& reg) {
+  return FH_.getRegularisation(reg);
+}
+
+void FactorHiGHSSolver::freeASmemory() {
+  // Give up memory used for AS.
+  freeVector(ptrAS_);
+  freeVector(rowsAS_);
+  freeVector(valAS_);
+}
+
+void FactorHiGHSSolver::freeNEmemory() {
+  // Give up memory used for NE.
+  freeVector(ptrNE_);
+  freeVector(rowsNE_);
+  freeVector(valNE_);
+  freeVector(ptrA_rw_);
+  freeVector(idxA_rw_);
+  freeVector(corr_A_);
+}
+
 }  // namespace hipo
