@@ -13,7 +13,7 @@
 namespace ipx {
 
 Int LpSolver::LoadModel(Int num_var, const double offset,
-			const double* obj, const double* lb,
+                        const double* obj, const double* lb,
                         const double* ub, Int num_constr, const Int* Ap,
                         const Int* Ai, const double* Ax, const double* rhs,
                         const char* constr_type) {
@@ -57,24 +57,24 @@ Int LpSolver::Solve() {
     control_.hLog("IPX version 1.0\n");
     try {
         InteriorPointSolve();
-	const bool run_crossover_on = control_.run_crossover() == 1;
-	const bool run_crossover_choose = control_.run_crossover() == -1;
-	const bool run_crossover_not_off = run_crossover_choose || run_crossover_on;
-	const bool run_crossover =
-	  (info_.status_ipm == IPX_STATUS_optimal && run_crossover_on) ||
-	  (info_.status_ipm == IPX_STATUS_imprecise && run_crossover_not_off);
-	//        if ((info_.status_ipm == IPX_STATUS_optimal ||
-	//             info_.status_ipm == IPX_STATUS_imprecise) && run_crossover_on) {
-	if (run_crossover) {
-	    if (run_crossover_on) {
-	      control_.hLog("Running crossover as requested\n");
-	    } else if (run_crossover_choose) {
-	      assert(info_.status_ipm == IPX_STATUS_imprecise);
-	      control_.hLog("Running crossover since IPX is imprecise\n");
-	    } else {
-	      assert(run_crossover_on || run_crossover_choose);
-	    }
-	    BuildCrossoverStartingPoint();
+        const bool run_crossover_on = control_.run_crossover() == 1;
+        const bool run_crossover_choose = control_.run_crossover() == -1;
+        const bool run_crossover_not_off = run_crossover_choose || run_crossover_on;
+        const bool run_crossover =
+          (info_.status_ipm == IPX_STATUS_optimal && run_crossover_on) ||
+          (info_.status_ipm == IPX_STATUS_imprecise && run_crossover_not_off);
+        //        if ((info_.status_ipm == IPX_STATUS_optimal ||
+        //             info_.status_ipm == IPX_STATUS_imprecise) && run_crossover_on) {
+        if (run_crossover) {
+            if (run_crossover_on) {
+              control_.hLog("Running crossover as requested\n");
+            } else if (run_crossover_choose) {
+              assert(info_.status_ipm == IPX_STATUS_imprecise);
+              control_.hLog("Running crossover since IPX is imprecise\n");
+            } else {
+              assert(run_crossover_on || run_crossover_choose);
+            }
+            BuildCrossoverStartingPoint();
             RunCrossover();
         }
         if (basis_) {
@@ -381,11 +381,10 @@ void LpSolver::InteriorPointSolve() {
             info_.rel_dresidual > control_.ipm_feasibility_tol())
             info_.status_ipm = IPX_STATUS_imprecise;
     }
-    if (info_.centring_tried) {
-      // Assess the success of analytic centre calculation
+
+    // Assess the success of analytic centre calculation
+    if (info_.centring_tried)
       info_.status_ipm = info_.centring_success ? IPX_STATUS_optimal : IPX_STATUS_imprecise;
-      assert(info_.status_ipm == IPX_STATUS_optimal);
-    }
 }
 
 void LpSolver::RunIPM() {
@@ -527,7 +526,7 @@ void LpSolver::BuildStartingBasis() {
     StartingBasis(iterate_.get(), basis_.get(), &info_);
     if (info_.errflag == IPX_ERROR_user_interrupt) {
         info_.errflag = 0;
-	info_.status_ipm = IPX_STATUS_user_interrupt;
+        info_.status_ipm = IPX_STATUS_user_interrupt;
         return;
     } else if (info_.errflag == IPX_ERROR_time_interrupt) {
         info_.errflag = 0;
