@@ -261,6 +261,8 @@ void reportOption(FILE* file, const HighsLogOptions& report_log_options,
                   const bool report_only_deviations,
                   const HighsFileType file_type);
 
+const string kHighsRunLogFile = "Highs.log";
+
 const string kSimplexString = "simplex";
 const string kIpmString = "ipm";
 const string kHipoString = "hipo";
@@ -307,6 +309,12 @@ const string kHipoTreeString = "tree";
 const string kHipoNodeString = "node";
 const string kHipoBothString = "both";
 
+// Strings for HiPO matrix reordering
+const string kHipoOrderingString = "hipo_ordering";
+const string kHipoMetisString = "metis";
+const string kHipoAmdString = "amd";
+const string kHipoRcmString = "rcm";
+
 struct HighsOptionsStruct {
   // Run-time options read from the command line
   std::string presolve;
@@ -371,8 +379,8 @@ struct HighsOptionsStruct {
   HighsInt ipm_iteration_limit;
   std::string hipo_system;
   std::string hipo_parallel_type;
+  std::string hipo_ordering;
   HighsInt hipo_block_size;
-  bool hipo_metis_no2hop;
 
   // Options for PDLP solver
   bool pdlp_scaling;
@@ -475,7 +483,6 @@ struct HighsOptionsStruct {
   double mip_min_logging_interval;
   std::string mip_lp_solver;
   std::string mip_ipm_solver;
-
 #ifdef HIGHS_DEBUGSOL
   std::string mip_debug_solution_file;
 #endif
@@ -484,6 +491,7 @@ struct HighsOptionsStruct {
   std::string mip_improving_solution_file;
   bool mip_root_presolve_only;
   HighsInt mip_lifting_for_probing;
+  bool mip_allow_cut_separation_at_nodes;
 
   // Logging callback identifiers
   HighsLogOptions log_options;
@@ -543,8 +551,8 @@ struct HighsOptionsStruct {
         ipm_iteration_limit(0),
         hipo_system(""),
         hipo_parallel_type(""),
+        hipo_ordering(""),
         hipo_block_size(0),
-        hipo_metis_no2hop(false),
         pdlp_scaling(false),
         pdlp_iteration_limit(0),
         pdlp_e_restart_method(0),
@@ -631,15 +639,18 @@ struct HighsOptionsStruct {
         mip_heuristic_run_zi_round(false),
         mip_heuristic_run_shifting(false),
         mip_min_logging_interval(0.0),
+        mip_lp_solver(""),
+        mip_ipm_solver(""),
 #ifdef HIGHS_DEBUGSOL
         mip_debug_solution_file(""),
 #endif
         mip_improving_solution_save(false),
         mip_improving_solution_report_sparse(false),
-        // clang-format off
         mip_improving_solution_file(""),
         mip_root_presolve_only(false),
-        mip_lifting_for_probing(-1) {};
+        mip_lifting_for_probing(-1),
+        // clang-format off
+        mip_allow_cut_separation_at_nodes(true) {};
   // clang-format on
 };
 
@@ -1206,6 +1217,12 @@ class HighsOptions : public HighsOptionsStruct {
                                        &mip_heuristic_run_shifting, false);
     records.push_back(record_bool);
 
+    record_bool = new OptionRecordBool(
+        "mip_allow_cut_separation_at_nodes",
+        "Whether cut separation at nodes is permitted", advanced,
+        &mip_allow_cut_separation_at_nodes, true);
+    records.push_back(record_bool);
+
     record_double = new OptionRecordDouble(
         "mip_rel_gap",
         "Tolerance on relative gap, |ub-lb|/|ub|, to determine whether "
@@ -1262,16 +1279,18 @@ class HighsOptions : public HighsOptionsStruct {
                                advanced, &hipo_parallel_type, kHipoBothString);
     records.push_back(record_string);
 
+    record_string =
+        new OptionRecordString(kHipoOrderingString,
+                               "HiPO matrix reordering option: \"metis\", "
+                               "\"amd\", \"rcm\" or \"choose\".",
+                               advanced, &hipo_ordering, kHighsChooseString);
+    records.push_back(record_string);
+
     record_int = new OptionRecordInt(
         "hipo_block_size", "Block size for dense linear algebra within HiPO",
         advanced, &hipo_block_size, 0, 128, kHighsIInf);
     records.push_back(record_int);
 
-    record_bool =
-        new OptionRecordBool("hipo_metis_no2hop", "Use option no2hop in Metis",
-                             advanced, &hipo_metis_no2hop, false);
-    records.push_back(record_bool);
-
     record_bool = new OptionRecordBool(
         "pdlp_scaling", "Scaling option for PDLP solver: Default = true",
         advanced, &pdlp_scaling, true);
