@@ -1694,3 +1694,30 @@ HighsDebugStatus HEkk::debugSimplexDualInfeasible(const std::string message,
         info.sum_dual_infeasibilities);
   return HighsDebugStatus::kOk;
 }
+
+bool HEkk::debugNoShiftsOrPerturbations() const {
+  switch (model_status_) {
+    case HighsModelStatus::kOptimal: {
+      if (this->info_.costs_shifted || this->info_.costs_perturbed ||
+          this->info_.bounds_shifted || this->info_.bounds_perturbed) {
+        return false;
+      }
+      break;
+    }
+    case HighsModelStatus::kInfeasible: {
+      if (this->info_.bounds_shifted || this->info_.bounds_perturbed) {
+        return false;
+      }
+      break;
+    }
+    case HighsModelStatus::kUnboundedOrInfeasible: {
+      if (this->info_.costs_shifted || this->info_.costs_perturbed) {
+        return false;
+      }
+      break;
+    }
+    default:
+      return true;
+  }
+  return true;
+}
